@@ -2,7 +2,8 @@
  * GUI and console interface.
  * Released under the GPL.
  * copyright Maher Awamy <muhri@muhri.net>
- * 	     Aaron Lehman <aaronl@vitelus.com> */
+ * 	     Aaron Lehmann <aaronl@vitelus.com>
+ * 	     Joshua Kwan <joshk@triplehelix.org> */
 
 #include "expand.xpm"
 #include "switch.h"
@@ -29,6 +30,10 @@ static short fgrep_gtkrc_for (gchar *needle)
 	FILE *gtkrc = fopen(path_to_gtkrc, "r");
 	char tempbuf[16384], *commentsearch;
 	g_free (path_to_gtkrc);
+
+        if (!gtkrc)
+          return 0;
+
 	while (!feof (gtkrc))
 	{
 		fgets (tempbuf, 16383, gtkrc);
@@ -62,12 +67,15 @@ get_dirs (void)
 	{
 		while ((dent = readdir (dir)))
 		{
+                        gchar* gtkrc_path = g_strconcat(dent->d_name, "/gtk-2.0/gtkrc", NULL);
 			stat(dent->d_name,&stats);
 			if (!S_ISDIR(stats.st_mode)) continue;
 			if (strcmp(dent->d_name,"." ) == 0) continue;
 			if (strcmp(dent->d_name,"..") == 0) continue;
+                        if (access(gtkrc_path, R_OK) == -1) continue;
 			g_hash_table_insert (hash, dent->d_name, dirname);
 			list = g_list_insert_sorted(list, g_strdup(dent->d_name), (GCompareFunc)strcmp);
+                        g_free(gtkrc_path);
 		}
 	}
  
@@ -82,12 +90,15 @@ get_dirs (void)
 	{
 		while ((dent = readdir (dir)))
 		{
+                        gchar* gtkrc_path = g_strconcat(dent->d_name, "/gtk-2.0/gtkrc", NULL);
 			stat(dent->d_name,&stats);
 			if (!S_ISDIR(stats.st_mode)) continue;
 			if (strcmp(dent->d_name, "." ) == 0) continue;
 			if (strcmp(dent->d_name, "..") == 0) continue;
+                        if (access(gtkrc_path, R_OK) == -1) continue;
 			g_hash_table_insert (hash, dent->d_name, dirname);
 			list =  g_list_insert_sorted(list, g_strdup(dent->d_name), (GCompareFunc)strcmp);
+                        g_free(gtkrc_path);
 		}
 	}
 	
@@ -353,22 +364,25 @@ dock (void)
      	gdk_pixmap_unref(gdkpixmap);
      	gdk_bitmap_unref(gdkbitmap);
      	evbox = gtk_event_box_new();
+
      	gtk_tooltips_set_tip(tips,evbox,"click here for more options","private");
      	gtk_widget_set_events(evbox, GDK_BUTTON_PRESS);
      	gtk_signal_connect(GTK_OBJECT(evbox), "button_press_event", GTK_SIGNAL_FUNC(on_eventbox_click), NULL);
      
      	gtk_container_add(GTK_CONTAINER(evbox), pixmap);
      	gtk_box_pack_start(GTK_BOX(box),evbox,FALSE,FALSE,FALSE);
-	
+	gtk_widget_show_all(box);
+     
      	box = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dockwin)->vbox), box, FALSE, FALSE, 0);
 
 	use_font_button = gtk_check_button_new_with_label("Use font: ");
 	gtk_box_pack_start(GTK_BOX(box), use_font_button, FALSE, FALSE, 0);
-
+	gtk_widget_show(use_font_button);
+     
 	font_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(box), font_entry, TRUE, TRUE, 0);
-
+	gtk_widget_show(font_entry);
 	if (newfont)
 	{
 		gtk_entry_set_text (GTK_ENTRY(font_entry), newfont);
@@ -397,21 +411,23 @@ dock (void)
 	browse = gtk_button_new_with_label("Browse...");
 	gtk_signal_connect(GTK_OBJECT(browse), "clicked", GTK_SIGNAL_FUNC(font_browse_clicked), NULL);
 	gtk_box_pack_start(GTK_BOX(box), browse, FALSE, FALSE, 0);
-
+	
 	button = gtk_button_new_with_label("Apply");
 	gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(apply_clicked),NULL);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG (dockwin)->action_area),button,TRUE,TRUE,FALSE);
-
+	gtk_widget_show(button);
+     
 	button = gtk_button_new_with_label("Preview");
 	gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(preview_clicked),NULL);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG (dockwin)->action_area),button,TRUE,TRUE,FALSE);
-
+	gtk_widget_show(button);
+     
 	install_button = gtk_button_new_with_label("Install New Theme");
 	gtk_signal_connect(GTK_OBJECT(install_button), "clicked", GTK_SIGNAL_FUNC(install_clicked), NULL);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG (dockwin)->action_area),install_button,TRUE,TRUE,FALSE);
 
-	gtk_widget_show_all(dockwin);
-	hide_stuff();
+	gtk_widget_show(dockwin);
+
 }
 
 void on_eventbox_click()
@@ -495,9 +511,9 @@ preview (gchar *rc_file)
 	     
 		while (gtk_events_pending())
 	     		gtk_main_iteration();
-		
+		usleep(50000);
 	}
-     
+            
      	pclose(pipe);
 	g_free (line);
 	g_free (path);
@@ -681,7 +697,7 @@ static short install_tarball (gchar *path, gchar **rc_file)
 		command = g_strdup_printf ("tar --directory %s -xzf %s 2>/dev/null", themedir, path);
 
 	/* Ensure that ~/.themes exists */
-	mkdir (themedir, 755);
+	mkdir (themedir, S_IRUSR | S_IWUSR | S_IXUSR);
 
 	result = system(command);
 	g_free (command);
