@@ -18,9 +18,15 @@ static void apply_clicked(GtkWidget *button, gpointer data);
 static void usage(void);
 static void ok_clicked(gchar *rc);
 static void preview_ok_clicked(gchar *rc);
-static void clist_insert(GtkWidget *clist);
+#ifdef SWITCH_GTK2
+static GtkTreeModel *create_model (void);
+void clist_insert (GtkTreeView *clist);
+#endif
 static void dock(void);
+#ifndef SWITCH_GTK2
 static int rightclick (GtkWidget *w, GdkEventButton *event, gpointer data);
+static void clist_insert(GtkWidget *clist);
+#endif
 static void preview(gchar *rc_file);
 static void preview_window(gchar *rc_file);
 static void send_refresh_signal(void);
